
#include "CSimNeut.hh"
#include "CSimulationTools.hh"
#include "CNeutData.hh"

#include "TROOT.h"
#include "TSystem.h"
#include "TStyle.h"
#include "TFile.h"
#include "TDirectory.h"
#include "TTree.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TF1.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"
#include "TGraph2DErrors.h"
#include "TVector3.h"
#include "TRandom.h"
#include "TBrowser.h"
#include "TPad.h"
#include "TCanvas.h"

#include <iostream>
#include <string>
#include <sstream>
#include <fstream>
#include <cmath>
#include <vector>
#include <deque>

// #include "CKinema.hh"

#include "TSimulationDataBeam.hh"
#include "TSimulationDataScintillator.hh"
#include "TSimulationParameterNeutronDetector.hh"

typedef CNeutData data_t;

CSimNeut::CSimNeut() : fOutputFileName("temp.root"), fNumberOfEntries(-1), fEventNumber(-1)
{;}

CSimNeut::~CSimNeut()
{;}

int CSimNeut::book(std::string rootfile)
{
  fROOTFileNameArray.push_back(rootfile);

  return 1;
}

int CSimNeut::SetOutputFileName(std::string rootfile)
{
  fOutputFileName = rootfile;

  return 1;
}

int CSimNeut::StartAnalysis(Int_t numberOfNeutrons, Int_t vetoMode, Int_t numberOfEntries)
{
  fNumberOfEntries = numberOfEntries;
  fNumberOfNeutrons = numberOfNeutrons;
  fVetoMode = vetoMode;

  fFileExp = TFile::Open(fOutputFileName.c_str(),"recreate");
  if(!fFileExp) return 0;
  fTopDirectory = gDirectory;

  while(!fROOTFileNameArray.empty()){
    std::string rootFileName = fROOTFileNameArray.front();
    fROOTFileNameArray.pop_front();

    std::cout << " analysis " << rootFileName << std::endl;
    fFileSim = TFile::Open(rootFileName.c_str(), "READ");
    if(!fFileSim) return 0;

    fTopDirectory->cd();

    ReadParameters();
    
    PrintParameters();

    // done only at first time
    static int initial = 1;
    if(initial){
      CreateHist();
      initial = 0;
    }

    CreateAndAnalyzeExpData();

    fFileSim->Close();
  }

  fFileExp->Write();
  fFileExp->Close();

  return 1;
}

int CSimNeut::ReadParameters()
{
  std::cout << "reading CSimNeut parameters" << std::endl;

  TSimulationParameterNeutronDetector* simulationParameterNeutronDetector;
  fFileSim->GetObject("parameterNeutronDetector", simulationParameterNeutronDetector);

  fSimPara = *simulationParameterNeutronDetector;

  fTotalNeutNumber = fSimPara.fNeutNumber*fSimPara.fNeutLayerNumber*fSimPara.fStuckNumber;
  fTotalVetoNumber = fSimPara.fVetoNumber*1*fSimPara.fStuckNumber;
  fTotalModuleNumber = fTotalNeutNumber + fTotalVetoNumber;

  fNeutHardwareThreshold = 2; // MeVee
  fVetoHardwareThreshold = 0.1; // MeVee
  fNeutSoftwareThreshold = 6; // MeVee
  fVetoSoftwareThreshold = 1; // MeVee
  fTimeResolution = 0.2*sqrt(2); // ns
  fNeutAttenuationLength = 668; // cm
  fVetoAttenuationLength = 258; // cm
  fPropagationVelocity = 15.8; // cm/ns
  fMultiplicity = 1;

  fAnalysisMethod = "single";
  //  fAnalysisMethod = "track";

  fStandardKineticEenrgy = 250; // standard incident energy
  //  fStandardBeta = 0.613314; // 250 MeV
  fStandardBeta = 0.56587; // 200 MeV
  
  TVector3  fNeutPos = TVector3(0, -(fSimPara.fNeutSize.y()+fSimPara.fNeutGap)*(fSimPara.fNeutNumber-1)/2, 0);
  std::cout << fSimPara.fNeutSize.y() << " " << fSimPara.fNeutGap << " " << fSimPara.fNeutNumber << std::endl;
  TVector3  fNeutPosShift = TVector3(0, fSimPara.fNeutSize.y()+fSimPara.fNeutGap, 0);
  TVector3  fVetoPos = TVector3(0,
				-(fSimPara.fVetoSize.y()+fSimPara.fVetoGap.y())*(fSimPara.fVetoNumber-1)/2,
				fSimPara.fVetoGap.z()/2);
  TVector3 fVetoPosShift = TVector3(0, fSimPara.fVetoSize.y()+fSimPara.fVetoGap.y(), fSimPara.fVetoGap.z());

  TVector3 fNeutLayerSize = TVector3(fSimPara.fNeutSize.x(),
				     (fSimPara.fNeutSize.y()+fSimPara.fNeutGap)*fSimPara.fNeutNumber-fSimPara.fNeutGap,
				     fSimPara.fNeutSize.z());
  TVector3 fVetoLayerSize = TVector3(fSimPara.fVetoSize.x(),
				     (fSimPara.fVetoSize.y()+fSimPara.fVetoGap.y())*fSimPara.fVetoNumber-fSimPara.fVetoGap.y(),
				     fSimPara.fVetoSize.z()+std::abs(fSimPara.fVetoGap.z()));
  double fNeutStuckSizeZ = (fNeutLayerSize.z()+fSimPara.fNeutLayerGap)*fSimPara.fNeutLayerNumber-fSimPara.fNeutLayerGap;
  double fVetoStuckSizeZ = fVetoLayerSize.z()+fSimPara.fNeutVetoGap;
  TVector3 fNeutLayerPos = TVector3(0, 0, -(fNeutStuckSizeZ-fVetoStuckSizeZ-fSimPara.fNeutSize.z())/2);
  TVector3 fVetoLayerPos = TVector3(0, 0, -fNeutStuckSizeZ/2);
  TVector3 fNeutLayerPosShift = TVector3(0, 0, fNeutLayerSize.z()+fSimPara.fNeutLayerGap);

  double fNeutLongSide = std::max(fNeutLayerSize.x(), fNeutLayerSize.y());
  double fVetoLongSide = std::max(fVetoLayerSize.x(), fVetoLayerSize.y());
  TVector3 fStuckSize = TVector3(std::max(fNeutLongSide, fVetoLongSide),
				 std::max(fNeutLongSide, fVetoLongSide),
				 fNeutStuckSizeZ+fVetoStuckSizeZ);
  TVector3 fStuckPos = TVector3(fSimPara.fPosition.x()+0.01,
				fSimPara.fPosition.y()+0.01,
				fSimPara.fPosition.z()+(fNeutStuckSizeZ-fVetoStuckSizeZ)/2); // neutの面
  TVector3 fStuckPosShift = TVector3(0, 0, fStuckSize.z()+fSimPara.fStuckGap);

//  std::cout << 0 << " " << -(fSimPara.fNeutSize.y()+fSimPara.fNeutGap)*(fSimPara.fNeutNumber-1)/2 << " " << 0 << " ";
//  std::cout << 0 << " " << fSimPara.fNeutSize.y()+fSimPara.fNeutGap << " " << 0 << " ";
//  std::cout << 0 << " " << -(fSimPara.fVetoSize.y()+fSimPara.fVetoGap.y())*(fSimPara.fVetoNumber-1)/2 << " " << fSimPara.fVetoGap.z()/2 << " ";
//  std::cout << 0 << " " << fSimPara.fVetoSize.y()+fSimPara.fVetoGap.y() << " " << fSimPara.fVetoGap.z() << " ";
//  std::cout << fSimPara.fNeutSize.x() << " " << (fSimPara.fNeutSize.y()+fSimPara.fNeutGap)*fSimPara.fNeutNumber-fSimPara.fNeutGap << " " << fSimPara.fNeutSize.z() << " ";
//  std::cout << fSimPara.fVetoSize.x() << " " << (fSimPara.fVetoSize.y()+fSimPara.fVetoGap.y())*fSimPara.fVetoNumber-fSimPara.fVetoGap.y() << " " << fSimPara.fVetoSize.z()+std::abs(fSimPara.fVetoGap.z()) << " ";
//  std::cout << (fNeutLayerSize.z()+fSimPara.fNeutLayerGap)*fSimPara.fNeutLayerNumber-fSimPara.fNeutLayerGap << " ";
//  std::cout << fVetoLayerSize.z()+fSimPara.fNeutVetoGap << " ";
//  std::cout << 0 << " " << 0 << " " << -(fNeutStuckSizeZ-fVetoStuckSizeZ-fSimPara.fNeutSize.z())/2 << " ";
//  std::cout << 0 << " " << 0 << " " << -fNeutStuckSizeZ/2 << " ";
//  std::cout << 0 << " " << 0 << " " << fNeutLayerSize.z()+fSimPara.fNeutLayerGap << " ";
//  std::cout << std::max(fNeutLayerSize.x(), fNeutLayerSize.y()) << " ";
//  std::cout << std::max(fVetoLayerSize.x(), fVetoLayerSize.y()) << " ";
//  std::cout << std::max(fNeutLongSide, fVetoLongSide) << " " << std::max(fNeutLongSide, fVetoLongSide) << " " << fNeutStuckSizeZ+fVetoStuckSizeZ << " ";
//  std::cout << fSimPara.fPosition.x()+0.01 << " " << fSimPara.fPosition.y()+0.01 << " " << fSimPara.fPosition.z()+(fNeutStuckSizeZ-fVetoStuckSizeZ)/2 << " ";
//  std::cout << 0 << " " <<  0 << " " << fStuckSize.z()+fSimPara.fStuckGap << " ";

  return 1;
}

int CSimNeut::PrintParameters()
{
  std::cout << "TotalNeutNumber: " << fTotalNeutNumber << "\n"
	    << "TotalVetoNumber: " << fTotalVetoNumber << "\n"
	    << "TotalModuleNumber: " << fTotalModuleNumber << "\n"

	    << "NeutHardwareThreshold: " << fNeutHardwareThreshold << "\n"

	    << "VetoHardwareThreshold: " << fVetoHardwareThreshold << "\n"
	    << "NeutSoftwareThreshold: " << fNeutSoftwareThreshold << "\n"
	    << "VetoSoftwareThreshold: " << fVetoSoftwareThreshold << "\n"
	    << "TimeResolution: " << fTimeResolution << "\n"

	    << "NeutAttenuationLength: " << fNeutAttenuationLength << "\n"
	    << "VetoAttenuationLength: " << fVetoAttenuationLength << "\n"
	    << "PropagationVelocity: " << fPropagationVelocity << "\n"
	    << "Multiplicity: " << fMultiplicity << "\n"
	    << "fAnalysisMethod: " << fAnalysisMethod << "\n"

	    << "fStandardKineticEenrgy: " << fStandardKineticEenrgy << "\n"
	    << "fStandardBeta: " << fStandardBeta << "\n"
	    << fSimPara << "\n"
    	    << std::endl;

  return 1;
}

int CSimNeut::CreateHist()
{
  fHIDQL = new TH2D("hidql", "id vs qu", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, 0, 150);
  fHIDQR = new TH2D("hidqr", "id vs qd", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, 0, 150);
  fHIDTL = new TH2D("hidtl", "id vs tu", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 600, 0, 100);
  fHIDTR = new TH2D("hidtr", "id vs td", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 600, 0, 100);
  fHIDQAve = new TH2D("hidqave", "id vs qave", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, 0, 150);
  fHIDTAve = new TH2D("hidtave", "id vs tave", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 600, 0, 100);

  fhenergy = new TH1D("henergy", "neutron energy", 800, 0, 400);
  fhenergytheta = new TH2D("henergytheta", "neutron energy vs theta", 8000, 0, 400, 200, 0, 0.1);

  if(fNumberOfNeutrons == 2){
    fhenergy1n = new TH1D("henergy1n", "neutron energy 1n", 800, 0, 400);

    fhq1q2       = new TH2D("hq1q2", "q1 vs q2", 600, 0, 150, 600, 0, 150);

    fhdtq1      = new TH2D("hdtq1", "2n dt vs q1", 400, -40, 40, 600, 0, 150);
    fhdtq2      = new TH2D("hdtq2", "2n dt vs q2", 400, -40, 40, 600, 0, 150);
    fhdbetaq1   = new TH2D("hdbetaq1", "2n delta beta vs q1", 400, -2, 2, 600, 0, 150);
    fhdbetaq2   = new TH2D("hdbetaq2", "2n delta beta vs q2", 400, -2, 2, 600, 0, 150);
    fhivdbetaq1 = new TH2D("hivdbetaq1", "2n 1/(delta beta) vs q1", 400, -15, 15, 600, 0, 150);
    fhivdbetaq2 = new TH2D("hivdbetaq2", "2n 1/(delta beta) vs q2", 400, -15, 15, 600, 0, 150);
    fhbetaddbetaq1   = new TH2D("hbetaddbetaq1", "2n delta beta/beta*0.6 vs q1", 400, -2, 2, 600, 0, 150);
    fhbetaddbetaq2   = new TH2D("hbetaddbetaq2", "2n delta beta/beta*0.6 vs q2", 400, -2, 2, 600, 0, 150);
    fhivbetaddbetaq1   = new TH2D("ivhbetaddbetaq1", "2n 1/(delta beta/beta*0.6) vs q1", 400, -15, 15, 600, 0, 150);
    fhivbetaddbetaq2   = new TH2D("ivhbetaddbetaq2", "2n 1/(delta beta/beta*0.6) vs q2", 400, -15, 15, 600, 0, 150);
  }

  fHTQ.resize(fTotalModuleNumber, (TH1*)0);
  fHQT.resize(fTotalModuleNumber, (TH1*)0);
  fHXY.resize(fTotalModuleNumber, (TH1*)0);
  //    for(int i=0; i<fTotalModuleNumber+1; ++i){
  for(int i=0; i<2; ++i){
    {
      std::ostringstream name,title;
      name << "htq" << i;
      title << "t vs q id" << i;
      if(i == 0){
	fHTQ[i] = new TH2D(name.str().c_str(), title.str().c_str(), 1000,50,80, 300,0,150);
      }else{
	fHTQ[i] = new TH2D(name.str().c_str(), title.str().c_str(), 1000,-5,5, 300,0,150);
      }
    }
    {
      std::ostringstream name,title;
      name << "hqt" << i;
      title << "q vs t id" << i;
      if(i == 0){
	fHQT[i] = new TH2D(name.str().c_str(), title.str().c_str(), 300,0,150, 1000,50,80);
      }else{
	fHQT[i] = new TH2D(name.str().c_str(), title.str().c_str(), 300,0,150, 1000,-5,5);
      }
    }
  }
  for(int i=0; i<1; ++i){
    {
      std::ostringstream name,title;
      name << "hxy" << i;
      title << "x vs y id" << i;
      fHXY[i] = new TH2D(name.str().c_str(), title.str().c_str(), 400,-100,100, 400,-100,100);
    }
  }

  return 1;
}

int CSimNeut::CreateAndAnalyzeExpData()
{
  TTree* tree;
  fFileSim->GetObject("simTree", tree);

  std::vector<TSimulationDataScintillator>* simulationDataScintillatorArray = new std::vector<TSimulationDataScintillator>;
  TBranch* branchSimulationDataScintillatorArray = 0;
  if(tree->SetBranchAddress("neutronDetector",&simulationDataScintillatorArray,&branchSimulationDataScintillatorArray) < 0) return 0;

  if(fNumberOfEntries < 0) fNumberOfEntries = tree->GetEntriesFast();
  for(int i=0; i<fNumberOfEntries; i++) {
    if((i+1)%10000 == 0) std::cout << "\r converted events:" << i+1 << std::flush;
    fEventNumber = i;

    Long64_t entry = tree->LoadTree(i);
    branchSimulationDataScintillatorArray->GetEntry(entry);

    if(simulationDataScintillatorArray->size() == 0) continue;

    fID.clear();
    fQL.clear();
    fQR.clear();
    fTL.clear();
    fTR.clear();

    // --------------------- create experiment data ---------------------
    SimToExp(simulationDataScintillatorArray);

    // ---------------------- fill data container --------------------------
    std::vector<data_t> data_v;
    for(int j=0; j<(int)fID.size(); ++j){
      data_t data;
      data.id = fID[j];
      data.qu = fQL[j];
      data.qd = fQR[j];
      data.tu = fTL[j];
      data.td = fTR[j];
      if(data.id < 120){
	data.qave = sqrt(data.qu*data.qd)*exp(fSimPara.fNeutSize.X()/fNeutAttenuationLength);
	data.tave = (data.tu+data.td)/2-fSimPara.fNeutSize.X()/fPropagationVelocity/2;
      }else{
	data.qave = sqrt(data.qu*data.qd)*exp(fSimPara.fVetoSize.X()/fVetoAttenuationLength);
	data.tave = (data.tu+data.td)/2-fSimPara.fVetoSize.X()/fPropagationVelocity/2;
      }	
      data.dt = data.tu-data.td;

      Int_t sequentialID = data.id;
      std::vector<int> id(3);
      if(sequentialID < 121){
	sequentialID -= 1;
	id[0] = sequentialID%fSimPara.fNeutNumber;
	sequentialID = (sequentialID-id[0])/fSimPara.fNeutNumber;;
	id[1] = sequentialID%fSimPara.fNeutLayerNumber;
	sequentialID = (sequentialID-id[1])/fSimPara.fNeutLayerNumber;
	id[2] = sequentialID;
      }else{
	sequentialID -= 1 + fTotalNeutNumber; 
	id[0] = sequentialID%fSimPara.fVetoNumber;
	sequentialID = (sequentialID-id[0])/fSimPara.fVetoNumber;;
	id[1] = 0;
	id[2] = sequentialID;
      }

      if(sequentialID < 121){
	data.pos = fSimPara.fNeutPos + fSimPara.fNeutPosShift*id[0];
	data.pos += fSimPara.fNeutLayerPos + fSimPara.fNeutLayerPosShift*id[1];
      }else{
	data.pos.SetX(fSimPara.fVetoPosShift.x()*id[0]);
	data.pos.SetY(fSimPara.fVetoPosShift.y()*id[0]);
	if(id[0]%2){
	  data.pos.SetZ(-1*fSimPara.fVetoPosShift.z());
	}
	data.pos += fSimPara.fVetoLayerPos;
      }
      double y = data.pos.x();
      double x = data.pos.y();
      data.pos.SetX(x);
      data.pos.SetY(y);
      
      data.pos += fSimPara.fStuckPos + fSimPara.fStuckPosShift*id[2];
      data.pos.SetY(data.pos.y()+data.dt*fPropagationVelocity/2);
      //      std::cout << data.id << " " << id[0] << " " << id[1] << " " << id[2] << " " << data.pos.X() << " " << data.pos.Y() << " " << data.pos.Z() << std::endl;

      Double_t time = data.tave;
      TVector3 beta, gamma, p;
      for(Int_t i=0; i<3; ++i){
	beta[i] = data.pos[i]/time/vlight;
	gamma[i] = 1/sqrt(1-beta[i]*beta[i]);
	p[i] = mneutron*gamma[i]*beta[i];
      }
      data.time = time;
      data.betaabs = sqrt(beta[0]*beta[0]+beta[1]*beta[1]+beta[2]*beta[2]);
      data.pabs = sqrt(p[0]*p[0]+p[1]*p[1]+p[2]*p[2]);
      data.energy = sqrt(mneutron*mneutron + data.pabs*data.pabs) - mneutron;
      data.theta = atan(sqrt(p[0]*p[0]+p[1]*p[2])/p[2]);
      data.phi = atan(p[1]/p[0]);

      data_v.push_back(data);
    }

    // -------------------------- analyze data -------------------------------------
    CSimulationTools::threshold(data_v, fNeutSoftwareThreshold, fVetoSoftwareThreshold);
    if(data_v.empty()) continue;

    if(fNumberOfNeutrons == 1){
      if(CSimulationTools::veto(data_v, fVetoMode)) continue;
      CSimulationTools::killveto(data_v);
      CSimulationTools::hitmintime(data_v);
    }else if(fNumberOfNeutrons == 2){
      // select proper 1n hit in first layer
      std::vector<data_t> data_v_copy = data_v;
      if(CSimulationTools::veto(data_v_copy, 1)) continue;
      CSimulationTools::killveto(data_v_copy);
      CSimulationTools::hitmintime(data_v_copy);
      //      CSimulationTools::acceptance(data_v_copy, 0.04);
      if(data_v_copy.empty()) continue;
      if(!(data_v_copy[0].energy > 195 && data_v_copy[0].energy < 205)) continue;
      fhenergy1n->Fill(data_v_copy[0].energy);

      // select 2hit
      if(CSimulationTools::veto(data_v, 2)) continue;
      //      if(CSimulationTools::veto(data_v, 1)) continue;
      CSimulationTools::killveto(data_v);
      CSimulationTools::hitmintime2(data_v);
      if(data_v.size() != 2) continue;
    }   

    // cross talk fill
    if(fNumberOfNeutrons == 2){
      data_t n1 = data_v[0];
      data_t n2 = data_v[1];
      if(data_v[0].pos[2] > data_v[1].pos[2]){
	n1 = data_v[1];
	n2 = data_v[0];
      }

      double t1 = n1.tave;
      double t2 = n2.tave;
      double q1 = n1.qave;
      double q2 = n2.qave;
      TVector3 pos1 = n1.pos;
      TVector3 pos2 = n2.pos;

      double x1 = pos1[0];
      double y1 = pos1[1];
      double z1 = pos1[2];
      double x2 = pos2[0];
      double y2 = pos2[1];
      double z2 = pos2[2];
      double dx = x2-x1;
      double dy = y2-y1;
      double dz = z2-z1;
      double dr = sqrt(dx*dx+dy*dy+dz*dz);
      double dt = t2-t1;
      double dbeta = dr/dt/vlight;
      double betaddbeta = (t1 < t2) ? dbeta/n1.betaabs*0.6 : dbeta/n2.betaabs*0.6;
      double ivbetaddbeta = 1/betaddbeta;

      //      if(!(q1 > 60 && 1/dbeta > 1.4 && 1/dbeta < 2)) continue;
      //      static std::ofstream ofs("temp.txt");
      //      ofs << i << std::endl;
      // fill 
      fhq1q2->Fill(q1,q2);

      fhdtq1->Fill(dt, q1);
      fhdtq2->Fill(dt, q2);
      fhdbetaq1->Fill(dbeta, q1);
      fhdbetaq2->Fill(dbeta, q2);
      fhivdbetaq1->Fill(1/dbeta, q1);
      fhivdbetaq2->Fill(1/dbeta, q2);
      fhbetaddbetaq1->Fill(betaddbeta, q1);
      fhbetaddbetaq2->Fill(betaddbeta, q2);
      fhivbetaddbetaq1->Fill(ivbetaddbeta, q1);
      fhivbetaddbetaq2->Fill(ivbetaddbeta, q2);

      //      for(int i=0; i<data_v.size(); ++i){
      //	data_t data = data_v[i];
      //	int id = data.id;
      //	double energy = data_v[i].energy;
      //
      //	fhenergy->Fill(energy);
      //      }
    }

    // fill
    for(int j=0; j<(int)data_v.size(); ++j){
      Int_t id = data_v[j].id;
      Double_t qu = data_v[j].qu;
      Double_t qd = data_v[j].qd;
      Double_t tu = data_v[j].tu;
      Double_t td = data_v[j].td;
      Double_t qave = data_v[j].qave;
      Double_t tave = data_v[j].tave;
      TVector3 pos = data_v[j].pos;

      fHIDQL->Fill(id, qu);
      fHIDQR->Fill(id, qd);
      fHIDTL->Fill(id, tu);
      fHIDTR->Fill(id, td);
      fHIDQAve->Fill(id, qave);
      fHIDTAve->Fill(id, tave);
      fHXY[0]->Fill(pos[0],pos[1]);

      fHTQ[0]->Fill(tave, qave);
      fHQT[0]->Fill(qave, tave);

      //      if(fNumberOfNeutrons != 2){
      double energy = data_v[j].energy;
      double posx = pos[0]+ gRandom->Uniform(-6,6);
      double theta = atan(sqrt(posx*posx+pos[1]*pos[1])/pos[2]);
      fhenergy->Fill(energy);
      fhenergytheta->Fill(energy,theta);
      //      }

      double r = sqrt(pos[0]*pos[0]+pos[1]*pos[1]+pos[2]*pos[2]);
      double TOF = r/fStandardBeta/29.979;
      //      std::cout << TOF << " " << tave << std::endl;
      //      if(id < 30){
      fHTQ[1]->Fill(tave - TOF, qave);
      fHQT[1]->Fill(qave, tave - TOF);
      //      }
    }
  }

  tree->ResetBranchAddresses();

  return 1;
}
  
void CSimNeut::SimToExp(std::vector<TSimulationDataScintillator>* simulationDataScintillatorArray)
{
  for(int i=0; i<(int)simulationDataScintillatorArray->size(); ++i){
    TSimulationDataScintillator simulationDataScintillator = (*simulationDataScintillatorArray)[i];

//    static std::ifstream ifs("temp.txt");
//    static int eventNumber = -1;
//    if(eventNumber < fEventNumber){
//      ifs >> eventNumber;
//    }
//    if(eventNumber == fEventNumber){
//      std::cout << fEventNumber << " " << simulationDataScintillator << std::endl;
//    }

    std::vector<Int_t> id = simulationDataScintillator.fModuleID;
    Int_t sequentialID = -1;
    if(simulationDataScintillator.fModuleName == "Neut"){
      sequentialID = 1+id[0]+id[1]*fSimPara.fNeutNumber+id[2]*fSimPara.fNeutNumber*fSimPara.fNeutLayerNumber;
    }else if(simulationDataScintillator.fModuleName == "Veto"){
      sequentialID = 1+id[0]                           +id[2]*fSimPara.fVetoNumber                          +fTotalNeutNumber;
    }else{
      std::cerr << "ou ou 1" << std::endl;
    }
    //    std::cout << simulationDataScintillator.fModuleName << " "
    //	      << sequentialID << " " << id[0] << " " << id[1] << " " << id[2] << std::endl;

    Double_t moduleSizeX = 0;
    Int_t rotationMode = 0;
    Double_t attenuationLength = 0;
    if(simulationDataScintillator.fModuleName == "Neut"){
      moduleSizeX = fSimPara.fNeutSize.X();
      rotationMode = fSimPara.fNeutRotationMode;
      attenuationLength = fNeutAttenuationLength;
    }else if(simulationDataScintillator.fModuleName == "Veto"){
      moduleSizeX = fSimPara.fVetoSize.X();
      rotationMode = fSimPara.fVetoRotationMode;
      attenuationLength = fVetoAttenuationLength;
    }else{
      std::cerr << "ou ou 2" << std::endl;
    }

    Double_t xl = moduleSizeX/2.
      -(simulationDataScintillator.fPosition.X()-fSimPara.fPosition.X());
    Double_t xr = moduleSizeX/2.
      +(simulationDataScintillator.fPosition.X()-fSimPara.fPosition.X());
    Double_t yl = moduleSizeX/2.
      -(simulationDataScintillator.fPosition.Y()-fSimPara.fPosition.Y()); // left == up
    Double_t yr = moduleSizeX/2.
      +(simulationDataScintillator.fPosition.Y()-fSimPara.fPosition.Y()); // right == down

    Double_t energyDepositEE = CSimulationTools::MeVtoMeVee(simulationDataScintillator.fKineticEnergy,
							    simulationDataScintillator.fKineticEnergy
							    - simulationDataScintillator.fEnergyDeposit,
							    simulationDataScintillator.fParticleName.Data());

    Double_t tl, tr, ql, qr;
    if(rotationMode == 1){ // stuck horizontal
      tl = simulationDataScintillator.fTime + yl/fPropagationVelocity;
      tr = simulationDataScintillator.fTime + yr/fPropagationVelocity;
      ql = energyDepositEE*exp(-yl/attenuationLength);
      qr = energyDepositEE*exp(-yr/attenuationLength);
    }else if(rotationMode == 2){ // vertical, horizontal, vertical, ....
      if(id[0]%2){
	tl = simulationDataScintillator.fTime + yl/fPropagationVelocity;
	tr = simulationDataScintillator.fTime + yr/fPropagationVelocity;
	ql = energyDepositEE*exp(-yl/attenuationLength);
	qr = energyDepositEE*exp(-yr/attenuationLength);
      }else{
	tl = simulationDataScintillator.fTime + xl/fPropagationVelocity;
	tr = simulationDataScintillator.fTime + xr/fPropagationVelocity;
	ql = energyDepositEE*exp(-xl/attenuationLength);
	qr = energyDepositEE*exp(-xr/attenuationLength);
      }
    }else{ // stuck vertical
      tl = simulationDataScintillator.fTime + xl/fPropagationVelocity;
      tr = simulationDataScintillator.fTime + xr/fPropagationVelocity;
      ql = energyDepositEE*exp(-xl/attenuationLength);
      qr = energyDepositEE*exp(-xr/attenuationLength);
    }      

    bool flagFound = false;
    for(int j=0; j<(int)fID.size(); ++j){
      if(fID[j] == sequentialID){
	flagFound = true;
	if(fTL[j] > tl) fTL[j] = tl;
	if(fTR[j] > tr) fTR[j] = tr;
	fQL[j] += ql;
	fQR[j] += qr;
	break;
      }
    }
    if(!flagFound){
      fID.push_back(sequentialID);
      fTL.push_back(tl);
      fTR.push_back(tr);
      fQL.push_back(ql);
      fQR.push_back(qr);
    }
  }

  // hardware threshold
  for(int i=0; i<(int)fID.size(); ++i){
    Double_t threshold;
    if(fID[i] <= 120){
      threshold = fNeutHardwareThreshold;
    }else{
      threshold = fVetoHardwareThreshold;
    }
    if(fQL[i] < threshold || fQR[i] < threshold){
      fID.erase(fID.begin()+i);
      fTL.erase(fTL.begin()+i);
      fTR.erase(fTR.begin()+i);
      fQL.erase(fQL.begin()+i);
      fQR.erase(fQR.begin()+i);
      --i;
      continue;
    }
  }

  // time blur
  for(int i=0; i<(int)fID.size(); ++i){
    fTL[i] += gRandom->Gaus(0,fTimeResolution);
    fTR[i] += gRandom->Gaus(0,fTimeResolution);  
  }
}
