
#include "DetectorConstruction.hh"
#include "DetectorConstructionMessenger.hh"

#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVReplica.hh"

#include "G4RunManager.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

#include "ScintillatorSD.hh"
#include "G4SDManager.hh"
#include "G4VSensitiveDetector.hh"

#include "globals.hh"

#include "TSimulationParameterNeutronDetector.hh"

#include "TVector3.h"

#include <sstream>
#include <algorithm>

DetectorConstruction::DetectorConstruction()
  : fNeutNumber(1), fNeutSize(10*cm, 10*cm, 10*cm), fNeutGap(0), fNeutRotationMode(0),
    fVetoNumber(0), fVetoSize(0, 0, 0),fVetoGap(0, 0, 0), fVetoRotationMode(0),
    fNeutLayerNumber(1), fNeutLayerGap(0),
    fNeutVetoGap(0),
    fStuckNumber(1), fStuckGap(0),
    fPosition(0, 0, 10.*m),
    fLogicNeut(0), fLogicNeutLayer(0), fLogicStuck(0),
    fLogicVeto(0), fLogicVetoLayer(0), fLogicWorld(0),
    fScintillatorSD(0)
{
  fDetectorConstructionMessenger = new DetectorConstructionMessenger(this);

//  fNeutMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_lH2");
//  fVetoMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_lH2");
  fNeutMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_PLASTIC_SC_VINYLTOLUENE");
  fVetoMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_PLASTIC_SC_VINYLTOLUENE");
  fNeutLayerMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");
  fStuckMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");
  fVetoLayerMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");
  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");
//  fNeutLayerMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
//  fStuckMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
//  fVetoLayerMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
//  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
}

DetectorConstruction::~DetectorConstruction()
{
  delete fDetectorConstructionMessenger;
}

G4VPhysicalVolume* DetectorConstruction::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  // if not using Veto
  if(fVetoNumber == 0){
    fVetoSize.set(0, 0, 0);
    fVetoGap.set(0, 0, 0);
    fNeutVetoGap = 0;
  }

  // Neut LV
  G4ThreeVector neutPos(0, -(fNeutSize.y()+fNeutGap)*(fNeutNumber-1)/2, 0);
  G4ThreeVector neutPosShift(0, fNeutSize.y()+fNeutGap, 0);
  G4Box* solidNeut = new G4Box("Neut",
			       fNeutSize.x()/2,
			       fNeutSize.y()/2,
			       fNeutSize.z()/2);
  fLogicNeut = new G4LogicalVolume(solidNeut,fNeutMaterial,"Neut");

  // Veto LV
  G4ThreeVector vetoPos(0, -(fVetoSize.y()+fVetoGap.y())*(fVetoNumber-1)/2, fVetoGap.z()/2);
  G4ThreeVector vetoPosShift(0, fVetoSize.y()+fVetoGap.y(), fVetoGap.z()); // fVetoGap.z()/2 *= -1
  G4Box* solidVeto = 0;
  if(fVetoNumber != 0){ 
    solidVeto = new G4Box("Veto",
			  fVetoSize.x()/2,
			  fVetoSize.y()/2,
			  fVetoSize.z()/2);
    fLogicVeto = new G4LogicalVolume(solidVeto,fVetoMaterial,"Veto");
  }

  // Neut and Veto Layer LV
  G4ThreeVector neutLayerSize(fNeutSize.x(),
			      (fNeutSize.y()+fNeutGap)*fNeutNumber-fNeutGap,
			      fNeutSize.z());
  G4ThreeVector vetoLayerSize(fVetoSize.x(),
			      (fVetoSize.y()+fVetoGap.y())*fVetoNumber-fVetoGap.y(),
			      fVetoSize.z()+std::abs(fVetoGap.z()));
  G4double neutStuckSizeZ = (neutLayerSize.z()+fNeutLayerGap)*fNeutLayerNumber-fNeutLayerGap;
  G4double vetoStuckSizeZ = vetoLayerSize.z()+fNeutVetoGap;
  G4ThreeVector neutLayerPos(0, 0, -(neutStuckSizeZ-vetoStuckSizeZ-fNeutSize.z())/2);
  G4ThreeVector vetoLayerPos(0, 0, -neutStuckSizeZ/2);
  G4ThreeVector neutLayerPosShift(0, 0, neutLayerSize.z()+fNeutLayerGap);
  G4Box* solidNeutLayer = new G4Box("NeutLayer",
				    neutLayerSize.x()/2,
				    neutLayerSize.y()/2,
				    neutLayerSize.z()/2);
  fLogicNeutLayer = new G4LogicalVolume(solidNeutLayer,fNeutLayerMaterial,"NeutLayer");
  G4Box* solidVetoLayer = 0;
  if(fVetoNumber != 0){ 
    solidVetoLayer = new G4Box("VetoLayer",
			       vetoLayerSize.x()/2,
			       vetoLayerSize.y()/2,
			       vetoLayerSize.z()/2);
    fLogicVetoLayer = new G4LogicalVolume(solidVetoLayer,fVetoLayerMaterial,"VetoLayer");
  }

  // Neutron Detector Stuck LV
  G4double neutLongSide = std::max(neutLayerSize.x(), neutLayerSize.y());
  G4double vetoLongSide = std::max(vetoLayerSize.x(), vetoLayerSize.y());
  G4ThreeVector stuckSize(std::max(neutLongSide, vetoLongSide),
			  std::max(neutLongSide, vetoLongSide),
			  neutStuckSizeZ+vetoStuckSizeZ);
  G4ThreeVector stuckPos(fPosition.x()+0.1*mm,
			 fPosition.y()+0.1*mm,
			 fPosition.z()+(neutStuckSizeZ-vetoStuckSizeZ)/2); // neutの面
  G4ThreeVector stuckPosShift(0, 0, stuckSize.z()+fStuckGap);

  G4Box* solidStuck = new G4Box("Stuck",
				stuckSize.x()/2,
				stuckSize.y()/2,
				stuckSize.z()/2);
  fLogicStuck = new G4LogicalVolume(solidStuck,fStuckMaterial,"Stuck");

  // World LV
  G4ThreeVector worldSize(1.2*2*(std::abs(stuckPos.x())+stuckSize.x()/2),
			  1.2*2*(std::abs(stuckPos.y())+stuckSize.y()/2),
			  1.2*2*(std::abs(stuckPos.z())+stuckSize.z()/2
				 +(stuckSize.z()+fStuckGap)*(fStuckNumber-1)));
  G4Box* solidWorld = new G4Box("World",
			    worldSize.x()/2,
			    worldSize.y()/2,
			    worldSize.z()/2);
  fLogicWorld = new G4LogicalVolume(solidWorld,fWorldMaterial,"World");
  
  // Neut PV
  G4String neutName("Neut");
  if(fNeutGap == 0){
    new G4PVReplica(neutName,
		    fLogicNeut,
		    fLogicNeutLayer,
		    kYAxis,
		    fNeutNumber,
		    fNeutSize.y());
  }else{
    G4ThreeVector neutPosTemp = neutPos;
    for(G4int i = 0; i < fNeutNumber; ++i){
      new G4PVPlacement(0,
			neutPosTemp,
			fLogicNeut,
			neutName,
			fLogicNeutLayer,
			false,
			i);
      neutPosTemp += neutPosShift;
    }
  }    

  // Veto PV
  if(fVetoNumber != 0){
    G4String vetoName("Veto");
    if(fVetoGap.y() == 0 && fVetoGap.z() == 0){
      new G4PVReplica(vetoName,
		      fLogicVeto,
		      fLogicVetoLayer,
		      kYAxis,
		      fVetoNumber,
		      fVetoSize.y());
    }else{
      G4ThreeVector vetoPosTemp = vetoPos;
      for(G4int i = 0; i < fVetoNumber; ++i){
	new G4PVPlacement(0,
			  vetoPosTemp,
			  fLogicVeto,
			  vetoName,
			  fLogicVetoLayer,
			  false,
			  i);
	vetoPosShift.setZ(-1*vetoPosShift.z());
	vetoPosTemp += vetoPosShift;
      }
    }    
  }

  // Neut Mother PV
  G4String neutLayerName("NeutLayer");
  G4ThreeVector neutLayerPosTemp = neutLayerPos;
  for(G4int i = 0; i < fNeutLayerNumber; ++i){
    G4RotationMatrix armRot= G4RotationMatrix();
    if(fNeutRotationMode == 1){ 
      armRot.rotateZ(-90*deg);
    }else if(fNeutRotationMode == 2){
      armRot.rotateZ(-90*deg*(i%2));
    }
    new G4PVPlacement(G4Transform3D(armRot, neutLayerPosTemp),
		      fLogicNeutLayer,
		      neutLayerName,
		      fLogicStuck,
		      false,
		      i);
    neutLayerPosTemp += neutLayerPosShift;
  }

  // Veto Mother PV
  if(fVetoNumber != 0){
    G4String vetoLayerName("VetoLayer");
    {
      G4int i = 0;
      G4RotationMatrix armRot= G4RotationMatrix();
      if(fVetoRotationMode == 1){ 
	armRot.rotateZ(-90*deg);
      }
      new G4PVPlacement(G4Transform3D(armRot, vetoLayerPos),
			fLogicVetoLayer,
			vetoLayerName,
			fLogicStuck,
			false,
			i);
    }
  }

  // Module Grand-Mother PV
  G4String stuckName("Stuck");
  G4ThreeVector stuckPosTemp = stuckPos;
  for(G4int i = 0; i < fStuckNumber; ++i){
    new G4PVPlacement(0,
		      stuckPosTemp,
		      fLogicStuck,
		      stuckName,
		      fLogicWorld,
		      false,
		      i);
    stuckPosTemp += stuckPosShift;
  }

//  // --- other materials (additional) --------------------------------------------------------------
//  // target LV and SUSWindow LV
//  G4ThreeVector targetSize(4*cm, 4*cm, 2.04*cm);
//  G4ThreeVector susWindowSize(2*m, 1*m, 3/cos(30*deg)*mm);
//  G4ThreeVector targetPos(0, 0, -(6.8*m/2-targetSize.z()/4+susWindowSize.z()/4));
//  G4ThreeVector susWindowPos(0, 0, 6.8*m/2+targetSize.z()/4-susWindowSize.z()/4);
//
//  G4Material* targetMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Li");
//  G4Box* solidTarget = new G4Box("Target",
//				 targetSize.x()/2,
//				 targetSize.y()/2,
//				 targetSize.z()/2);
//  G4LogicalVolume* logicTarget = new G4LogicalVolume(solidTarget,targetMaterial,"Target");
//
//  G4Material* susWindowMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_STAINLESS-STEEL");
//  G4Box* solidSUSWindow = new G4Box("SUSWindow",
//				   susWindowSize.x()/2,
//				   susWindowSize.y()/2,
//				   susWindowSize.z()/2);
//  G4LogicalVolume* logicSUSWindow = new G4LogicalVolume(solidSUSWindow,susWindowMaterial,"SUSWindow");
//
//  // Vacuum LV
//  G4ThreeVector vacuumSize(susWindowSize.x(), susWindowSize.y(), (susWindowPos.z()-targetPos.z())+susWindowSize.z()/2+targetSize.z()/2);
//  G4ThreeVector vacuumPos(0, 0, vacuumSize.z()/2-targetSize.z()/2);
//  G4Material* vacuumMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
//  G4Box* solidVacuum = new G4Box("Vacuum",
//				 vacuumSize.x()/2,
//				 vacuumSize.y()/2,
//				 vacuumSize.z()/2);
//  G4LogicalVolume* logicVacuum = new G4LogicalVolume(solidVacuum,vacuumMaterial,"Vacuum");
//
//  // target PV
//  G4String targetName("Target");
//  new G4PVPlacement(0,
//		    targetPos,
//		    logicTarget,
//		    targetName,
//		    logicVacuum,
//		    false,
//		    0);
//
//  // SUSWindow PV
//  G4String susWindowName("SUSWindow");
//  new G4PVPlacement(0,
//		    susWindowPos,
//		    logicSUSWindow,
//		    susWindowName,
//		    logicVacuum,
//		    false,
//		    0);
//
//  // vacuum PV
//  G4String vacuumName("Vacuum");
//  new G4PVPlacement(0,
//		    vacuumPos,
//		    logicVacuum,
//		    vacuumName,
//		    fLogicWorld,
//		    false,
//		    0);
//
//  G4VisAttributes* targetColor = new G4VisAttributes(G4Colour::Red());
//  logicTarget->SetVisAttributes(targetColor);
//
//  G4VisAttributes* susWindowColor = new G4VisAttributes(G4Colour::Black());
//  logicSUSWindow->SetVisAttributes(susWindowColor);
//
//  G4VisAttributes* vacuumColor = new G4VisAttributes(G4Colour::Gray());
//  logicVacuum->SetVisAttributes(vacuumColor);
//  // ----------------------------------------------------------------------------------

  // World PV
  G4ThreeVector worldPos(0, 0, 0);  
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);

  // add SD
  if(!fScintillatorSD){
    G4SDManager* SDman = G4SDManager::GetSDMpointer();
    fScintillatorSD = new ScintillatorSD("/ScintillatorSD");
    SDman->AddNewDetector(fScintillatorSD);
  }
  fLogicNeut->SetSensitiveDetector(fScintillatorSD);
  if(fVetoNumber != 0) fLogicVeto->SetSensitiveDetector(fScintillatorSD);

  // colors
  fLogicWorld->SetVisAttributes(G4VisAttributes::Invisible);
  fLogicStuck->SetVisAttributes(G4VisAttributes::Invisible);
  fLogicNeutLayer->SetVisAttributes(G4VisAttributes::Invisible);
  if(fVetoNumber != 0) fLogicVetoLayer->SetVisAttributes(G4VisAttributes::Invisible);

  G4VisAttributes* neutColour = new G4VisAttributes(G4Colour::Yellow());
  fLogicNeut->SetVisAttributes(neutColour);
  G4VisAttributes* vetoColour = new G4VisAttributes(G4Colour::Cyan());
  if(fVetoNumber != 0) fLogicVeto->SetVisAttributes(vetoColour);
  
  // Print status
  G4cout << "### Neut consist of " 
         << fNeutMaterial->GetName() 
         <<  "  ###" << G4endl;

  G4cout <<* (G4Material::GetMaterialTable()) << G4endl;

  // save parameter which was used
  if(!gSimulationParameterNeutronDetector){
    gSimulationParameterNeutronDetector = new TSimulationParameterNeutronDetector("parameterNeutronDetector");
  }
  gSimulationParameterNeutronDetector->fNeutNumber = fNeutNumber;
  CopyG4ThreeVectorToTVector3(fNeutSize/cm, gSimulationParameterNeutronDetector->fNeutSize);
  gSimulationParameterNeutronDetector->fNeutGap = fNeutGap/cm;
  gSimulationParameterNeutronDetector->fNeutRotationMode = fNeutRotationMode;
  gSimulationParameterNeutronDetector->fVetoNumber = fVetoNumber;
  CopyG4ThreeVectorToTVector3(fVetoSize/cm, gSimulationParameterNeutronDetector->fVetoSize);
  CopyG4ThreeVectorToTVector3(fVetoGap/cm, gSimulationParameterNeutronDetector->fVetoGap);
  gSimulationParameterNeutronDetector->fVetoRotationMode = fVetoRotationMode;
  gSimulationParameterNeutronDetector->fNeutLayerNumber = fNeutLayerNumber;
  gSimulationParameterNeutronDetector->fNeutLayerGap = fNeutLayerGap/cm;
  gSimulationParameterNeutronDetector->fNeutVetoGap = fNeutVetoGap/cm;
  gSimulationParameterNeutronDetector->fStuckNumber = fStuckNumber;
  gSimulationParameterNeutronDetector->fStuckGap = fStuckGap/cm;
  CopyG4ThreeVectorToTVector3(fPosition/cm, gSimulationParameterNeutronDetector->fPosition);

  CopyG4ThreeVectorToTVector3(neutPos/cm, gSimulationParameterNeutronDetector->fNeutPos);
  CopyG4ThreeVectorToTVector3(neutPosShift/cm, gSimulationParameterNeutronDetector->fNeutPosShift);
  CopyG4ThreeVectorToTVector3(vetoPos/cm, gSimulationParameterNeutronDetector->fVetoPos);
  CopyG4ThreeVectorToTVector3(vetoPosShift/cm, gSimulationParameterNeutronDetector->fVetoPosShift);
  CopyG4ThreeVectorToTVector3(neutLayerSize/cm, gSimulationParameterNeutronDetector->fNeutLayerSize);
  CopyG4ThreeVectorToTVector3(vetoLayerSize/cm, gSimulationParameterNeutronDetector->fVetoLayerSize);
  gSimulationParameterNeutronDetector->fNeutStuckSizeZ = neutStuckSizeZ/cm;
  gSimulationParameterNeutronDetector->fVetoStuckSizeZ = vetoStuckSizeZ/cm;
  CopyG4ThreeVectorToTVector3(neutLayerPos/cm, gSimulationParameterNeutronDetector->fNeutLayerPos);
  CopyG4ThreeVectorToTVector3(vetoLayerPos/cm, gSimulationParameterNeutronDetector->fVetoLayerPos);
  CopyG4ThreeVectorToTVector3(neutLayerPosShift/cm, gSimulationParameterNeutronDetector->fNeutLayerPosShift);
  gSimulationParameterNeutronDetector->fNeutLongSide = neutLongSide/cm;
  gSimulationParameterNeutronDetector->fVetoLongSide = vetoLongSide/cm;
  CopyG4ThreeVectorToTVector3(stuckSize/cm, gSimulationParameterNeutronDetector->fStuckSize);
  CopyG4ThreeVectorToTVector3(stuckPos/cm, gSimulationParameterNeutronDetector->fStuckPos);
  CopyG4ThreeVectorToTVector3(stuckPosShift/cm, gSimulationParameterNeutronDetector->fStuckPosShift);

#ifdef DEBUG
  std::cout << " Parameters: " << *gSimulationParameterNeutronDetector << "\n" << std::endl;
#endif

  return world;
}

void DetectorConstruction::CopyG4ThreeVectorToTVector3(const G4ThreeVector& threeVector, TVector3& vector3)
{
  vector3.SetXYZ(threeVector.x(),
		 threeVector.y(),
		 threeVector.z());
}

void DetectorConstruction::UpdateGeometry()
{
  G4RunManager::GetRunManager()->DefineWorldVolume(Construct());
}

void DetectorConstruction::SetNeutNumber(G4int val)  
{
  if(val > 0 && val != fNeutNumber){
    fNeutNumber = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetNeutSizeX(G4double val)  
{
  if(val > 0 && val != fNeutSize.x()){
    fNeutSize.setX(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetNeutSizeY(G4double val)  
{
  if(val > 0 && val != fNeutSize.y()){
    fNeutSize.setY(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetNeutSizeZ(G4double val)  
{
  if(val > 0 && val != fNeutSize.z()){
    fNeutSize.setZ(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetNeutGap(G4double val)  
{
  if(val >= 0 && val != fNeutGap){
    fNeutGap = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetNeutRotationMode(G4int val)  
{
  if(val != fNeutRotationMode){
    fNeutRotationMode = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetVetoNumber(G4int val)  
{
  if(val >= 0 && val != fVetoNumber){
    fVetoNumber = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetVetoSizeX(G4double val)  
{
  if(val > 0 && val != fVetoSize.x()){
    fVetoSize.setX(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetVetoSizeY(G4double val)  
{
  if(val > 0 && val != fVetoSize.y()){
    fVetoSize.setY(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetVetoSizeZ(G4double val)  
{
  if(val > 0 && val != fVetoSize.z()){
    fVetoSize.setZ(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetVetoGapY(G4double val)  
{
  if(val != fVetoGap.y()){
    fVetoGap.setY(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetVetoGapZ(G4double val)  
{
  if(val != fVetoGap.z()){
    fVetoGap.setZ(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetVetoRotationMode(G4int val)  
{
  if(val != fVetoRotationMode){
    fVetoRotationMode = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetNeutLayerNumber(G4int val)  
{
  if(val > 0 && val != fNeutLayerNumber){
    fNeutLayerNumber = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetNeutLayerGap(G4double val)  
{
  if(val >= 0 && val != fNeutLayerGap){
    fNeutLayerGap = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetNeutVetoGap(G4double val)  
{
  if(val >= 0 && val != fNeutVetoGap){
    fNeutVetoGap = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetStuckNumber(G4int val)  
{
  if(val > 0 && val != fStuckNumber){
    fStuckNumber = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetStuckGap(G4double val)  
{
  if(val >= 0 && val != fStuckGap){
    fStuckGap = val;
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  } 
}

void DetectorConstruction::SetStuckInterval(G4double val)  
{
  G4double neutStuckSizeZ = (fNeutSize.z()+fNeutLayerGap)*fNeutLayerNumber-fNeutLayerGap;
  G4double vetoStuckSizeZ = fVetoSize.z()+fVetoGap.z()+fNeutVetoGap;

  G4double stuckGap = val - (neutStuckSizeZ + vetoStuckSizeZ);
  if(stuckGap < 0){
    G4cerr << "" << G4endl;
    G4cerr << " DetectorConstrucktion::SetStuckInterval> Stuck gap could not be less than 0 !!" << G4endl;
    G4cerr << " DetectorConstrucktion::SetStuckInterval> Set proper interval (or maybe better to set at the last)" << G4endl;
    G4cerr << " DetectorConstrucktion::SetStuckInterval> Current full width of stuck is "
	   << (neutStuckSizeZ + vetoStuckSizeZ)/cm << " cm"<< G4endl;
    G4cerr << "" << G4endl;
    G4Exception(0, 0, FatalException, 0);
  }else{
    SetStuckGap(stuckGap);
  }
}

void DetectorConstruction::SetPositionX(G4double val)  
{
  if(val != fPosition.x()){
    fPosition.setX(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  }
}

void DetectorConstruction::SetPositionY(G4double val)  
{
  if(val != fPosition.y()){
    fPosition.setY(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  }
}

void DetectorConstruction::SetPositionZ(G4double val)  
{
  if(val >= 0 && val != fPosition.z()){
    fPosition.setZ(val);
    G4RunManager::GetRunManager()->GeometryHasBeenModified();
  }
}

// void DetectorConstruction::SetNeutMaterial(const G4String& mat)
// {
//   G4Material* material = G4NistManager::Instance()->FindOrBuildMaterial(mat);
//   if(material && material != fNeutMaterial){
//     fNeutMaterial = material;
//     if(fLogicNeut) fLogicNeut->SetMaterial(fNeutMaterial);
//     G4RunManager::GetRunManager()->PhysicsHasBeenModified();
//   }
// }
// 
// void DetectorConstruction::SetWorldMaterial(const G4String& mat)
// {
//   G4Material* material = G4NistManager::Instance()->FindOrBuildMaterial(mat);
//   if(material && material != fWorldMaterial){
//     fWorldMaterial = material;
//     if(fLogicWorld) fLogicWorld->SetMaterial(fWorldMaterial);
//     G4RunManager::GetRunManager()->PhysicsHasBeenModified();
//   }
// }

